@@ -496,27 +496,30 @@ where
     IndexType: ArrowNumericType,
     IndexType::Native: ToPrimitive,
 {
-    // TODO optimize decimal take and construct decimal array from MutableBuffer
-    let mut builder = DecimalBuilder::new(
-        indices.len(),
-        decimal_values.precision(),
-        decimal_values.scale(),
-    );
-    for i in 0..indices.len() {
-        if indices.is_null(i) {
-            builder.append_null()?;
-        } else {
-            let index = ToPrimitive::to_usize(&indices.value(i)).ok_or_else(|| {
-                ArrowError::ComputeError("Cast to usize failed".to_string())
-            })?;
-            if decimal_values.is_null(index) {
-                builder.append_null()?
-            } else {
-                builder.append_value(decimal_values.value(index))?
-            }
-        }
-    }
-    Ok(builder.finish())
+    indices
+        .iter()
+        .map(|index| {
+            // Use type annotations below for readability (was blowing
+            // my mind otherwise)
+            let t: Option<Result<Option<_>>> = index.map(|index| {
+                let index = ToPrimitive::to_usize(&index).ok_or_else(|| {
+                    ArrowError::ComputeError("Cast to usize failed".to_string())
+                })?;
+
+                if decimal_values.is_null(index) {
+                    Ok(None)
+                } else {
+                    Ok(Some(decimal_values.value(index)))
+                }
+            });
+            let t: Result<Option<Option<_>>> = t.transpose();
+            let t: Result<Option<_>> = t.map(|t| t.flatten());
+            t
+        })
+        .collect::<Result<DecimalArray>>()?
+        // PERF: we could avoid re-validating that the data in
+        // DecimalArray was in range as we know it came from a valid DecimalArray
+        .with_precision_and_scale(decimal_values.precision(), decimal_values.scale())
 }
 
 /// `take` implementation for all primitive arrays
@@ -965,30 +968,19 @@ mod tests {
         precision: &usize,
         scale: &usize,
     ) -> Result<()> {
-        let mut builder = DecimalBuilder::new(data.len(), *precision, *scale);
-        for value in data {
-            match value {
-                None => {
-                    builder.append_null()?;
-                }
-                Some(v) => {
-                    builder.append_value(v)?;
-                }
-            }
-        }
-        let output = builder.finish();
-        let mut builder = DecimalBuilder::new(expected_data.len(), *precision, *scale);
-        for value in expected_data {
-            match value {
-                None => {
-                    builder.append_null()?;
-                }
-                Some(v) => {
-                    builder.append_value(v)?;
-                }
-            }
-        }
-        let expected = Arc::new(builder.finish()) as ArrayRef;
+        let output = data
+            .into_iter()
+            .collect::<DecimalArray>()
+            .with_precision_and_scale(*precision, *scale)
+            .unwrap();
+
+        let expected = expected_data
+            .into_iter()
+            .collect::<DecimalArray>()
+            .with_precision_and_scale(*precision, *scale)
+            .unwrap();
+
+        let expected = Arc::new(expected) as ArrayRef;
         let output = take(&output, index, options).unwrap();
         assert_eq!(&output, &expected);
         Ok(())
